package com.sztzjy.money_management.controller;

import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.nimbusds.jose.shaded.gson.Gson;
import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.entity.dto.*;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.service.CaseService;
import com.sztzjy.money_management.service.ScoreRankService;
import com.sztzjy.money_management.service.TrainingService;
import com.sztzjy.money_management.util.ResultEntity;
import com.sztzjy.money_management.util.excel.FilePortUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.math.BigDecimal;
import java.util.*;

@RestController
@Api(tags = "成绩相关")
@RequestMapping("api/GradeController")
public class GradeController {
    @Autowired
    StuTrainingMapper trainingMapper;
    @Autowired
    StudentScoreWeightMapper studentScoreWeightMapper;
    @Autowired
    TrainingReportMapper trainingReportMapper;
    @Autowired
    TrainingService trainingService;
    @Autowired
    StuTheoryRecordMapper stuTheoryRecordMapper;
    @Autowired
    TotalWeightMapper totalWeightMapper;
    @Autowired
    ScoreRankMapper scoreRankMapper;
    @Autowired
    ScoreRankService scoreRankService;
    @Autowired
    ClientWeightMapper clientWeightMapper;
    @Autowired
    ChapterMapper chapterMapper;
    @Value("${file.path}")
    private String filePath;
    @Autowired
    SynthesisPlanClientMapper synthesisPlanClientMapper;
    @Autowired
    CaseService caseService;

    @AnonymousAccess
    @ApiOperation("实训演练-学习成绩得分情况展示")
    @PostMapping("getScoreInfo")
    public ResultEntity<StuTrainingDto> getScoreInfo(@RequestParam String userId,
                                     @RequestParam String chapterId,
                                     @RequestParam String schoolId) {
        StudentScoreWeight studentScoreWeight=new StudentScoreWeight();
        //先根据schoolID和chapterId查出学习成绩权重表数据
        StudentScoreWeightExample studentScoreWeightExample = new StudentScoreWeightExample();
        studentScoreWeightExample.createCriteria().andSchoolIdEqualTo(schoolId).andChapterIdEqualTo(chapterId);
        List<StudentScoreWeight> studentScoreWeights = studentScoreWeightMapper.selectByExample(studentScoreWeightExample);
        if(studentScoreWeights.isEmpty()){  //设置默认权重与计分规则
            //计分规则
            studentScoreWeight.setSummaryOfKnowledgeScore(BigDecimal.valueOf(20));
            studentScoreWeight.setResourceLearningScore(BigDecimal.valueOf(2));
            studentScoreWeight.setLearningAssessmentScore(BigDecimal.valueOf(10));
            studentScoreWeight.setExperimentalTrainingScore(BigDecimal.valueOf(20));

            //权重
            studentScoreWeight.setSummaryOfKnowledgeWeight(BigDecimal.valueOf(0.2));
            studentScoreWeight.setResourceLearningWeight(BigDecimal.valueOf(0.2));
            studentScoreWeight.setLearningAssessmentWeight(BigDecimal.valueOf(0.2));
            studentScoreWeight.setExperimentalTrainingWeight(BigDecimal.valueOf(0.2));
            studentScoreWeight.setReportWeight(BigDecimal.valueOf(0.2));
        }else {
            studentScoreWeight = studentScoreWeights.get(0);
        }
        //再根据userid和chapterId查出学生实训表中的数据
        StuTrainingExample stuTrainingExample = new StuTrainingExample();
        stuTrainingExample.createCriteria().andUserIdEqualTo(userId).andChapterIdEqualTo(chapterId);
        List<StuTraining> stuTrainings = trainingMapper.selectByExample(stuTrainingExample);
        StuTraining stuTraining = stuTrainings.get(0);
        //进行整合
        StuTrainingDto stuTrainingDto = new StuTrainingDto();
        //设置返回对象计分规则
        stuTrainingDto.setKnowledgeSummaryDefaultRule(studentScoreWeight.getSummaryOfKnowledgeScore());
        stuTrainingDto.setResourceLearningDefaultRule(studentScoreWeight.getResourceLearningScore());
        stuTrainingDto.setLearningEvalDefaultRule(studentScoreWeight.getLearningAssessmentScore());
        stuTrainingDto.setExpTrainingDefautRule(studentScoreWeight.getExperimentalTrainingScore());
        //设置返回对象权重
        stuTrainingDto.setKnowledgeSummaryWeight(studentScoreWeight.getSummaryOfKnowledgeWeight());
        stuTrainingDto.setResourceLearningWeight(studentScoreWeight.getResourceLearningWeight());
        stuTrainingDto.setLearningEvalWeight(studentScoreWeight.getLearningAssessmentWeight());
        stuTrainingDto.setExpTrainingWeight(studentScoreWeight.getExperimentalTrainingWeight());
        stuTrainingDto.setReportWeight(studentScoreWeight.getReportWeight());
        //设置返回对象完成情况
        stuTrainingDto.setKnowledgeSummaryCompleteStatus(stuTraining.getKnowledgeSummaryCompleteStatus());
        stuTrainingDto.setResourceLearningCompleteStatus(stuTraining.getResourceLearningCompleteStatus());
        stuTrainingDto.setLearningEvalCompleteStatus(stuTraining.getLearningEvalCompleteStatus());
        stuTrainingDto.setExpTrainingCompleteStatus(stuTraining.getExpTrainingCompleteStatus());
        TrainingReportExample trainingReportExample = new TrainingReportExample();
        trainingReportExample.createCriteria().andChapterIdEqualTo(chapterId).andUserIdEqualTo(userId);
        //设置返回对象得分  根据学生实训表完成情况 设置得分
        stuTrainingDto.setKnowledgeSummaryScore(studentScoreWeight.getSummaryOfKnowledgeScore().multiply(BigDecimal.valueOf(stuTraining.getKnowledgeSummaryCompleteStatus())));
        stuTrainingDto.setResourceLearningScore(studentScoreWeight.getResourceLearningScore().multiply(BigDecimal.valueOf(stuTraining.getResourceLearningCompleteStatus())));
        stuTrainingDto.setLearningEvalScore(studentScoreWeight.getLearningAssessmentScore().multiply(BigDecimal.valueOf(stuTraining.getLearningEvalCompleteStatus())));
        stuTrainingDto.setCaseStuScore(studentScoreWeight.getExperimentalTrainingScore().multiply(BigDecimal.valueOf(stuTraining.getExpTrainingCompleteStatus())));

        //设置实验报告情况及得分
        List<TrainingReport> trainingReports = trainingReportMapper.selectByExample(trainingReportExample);
        if(trainingReports.isEmpty()){
            stuTrainingDto.setReportCompleteStatus("未提交");
            stuTrainingDto.setReportScore(BigDecimal.valueOf(0));
        }else {
            stuTrainingDto.setReportCompleteStatus("已提交");
            stuTrainingDto.setReportScore(trainingReports.get(0).getTeacherScore());
        }
        return new ResultEntity<>(HttpStatus.OK, "得分情况展示",stuTrainingDto);

    }

    @GetMapping("/returnTrainingId")
    @ApiOperation("第一次进入时创建学生实训对象")
    @AnonymousAccess
    public ResultEntity<String> returnTrainingId(@RequestParam String chapterId,
                                                 @RequestParam String userId) {
        StuTrainingExample stuTrainingExample = new StuTrainingExample();
        stuTrainingExample.createCriteria().andChapterIdEqualTo(chapterId).andUserIdEqualTo(userId);
        List<StuTrainingWithBLOBs> list = trainingMapper.selectByExampleWithBLOBs(stuTrainingExample);
        if (list.isEmpty()) { //该用户第一次进入这个章节的知识概要 新增
            StuTrainingWithBLOBs stuTraining = new StuTrainingWithBLOBs();
            String uuid = IdUtil.randomUUID();
            stuTraining.setTrainingId(uuid);
            stuTraining.setChapterId(chapterId);
            stuTraining.setUserId(userId);
            trainingMapper.insert(stuTraining);
            return new ResultEntity<>(uuid);
        } else {
            return new ResultEntity<>(list.get(0).getTrainingId());
        }
    }



    //学生端 客户案例成绩展示
    @AnonymousAccess
    @ApiOperation("学生端-成绩中心-客户案例成绩展示")
    @PostMapping("selectStuClientCaseScoreList")
    public ResultEntity<List<StuClientCaseScoreDto>> selectStuClientCaseScoreList(@RequestParam String userId,
                                                     @RequestParam String schoolId) {
        List<StuClientCaseScoreDto> list=trainingService.selectStuClientCaseScoreList(userId,schoolId);
        return new ResultEntity<>(HttpStatus.OK, "学生端-成绩中心-客户案例成绩展示",list);
    }

    //学生端 综合规划成绩
    @AnonymousAccess
    @ApiOperation("学生端-成绩中心-综合规划成绩展示")
    @PostMapping("selectStuSynthesisPlanScoreList")
    public ResultEntity<List<StuSynthesisPlanScoreDto>> selectStuSynthesisPlanScoreList(@RequestParam String userId,
                                                                                        @RequestParam String schoolId) {
        List<StuSynthesisPlanScoreDto> list=trainingService.selectStuSynthesisPlanScoreList(userId,schoolId);
        return new ResultEntity<>(HttpStatus.OK, "学生端-成绩中心-综合规划成绩展示",list);
    }

    //学生端 综合规划审核报告展示  同综合规划-规划报告-审核报告

    //学生端 理论考核成绩
    @AnonymousAccess
    @ApiOperation("学生端-成绩中心-理论考核成绩展示")
    @PostMapping("selectStuTheoryScore")
    public ResultEntity<StuTheoryRecordDto> selectStuTheoryRecord(@RequestParam String userId,@RequestParam String schoolId) {
        StuTheoryRecord stuTheoryRecord = stuTheoryRecordMapper.selectByPrimaryKey(userId);
        if(stuTheoryRecord==null){
            return new ResultEntity<>(HttpStatus.OK, "暂无数据",null);
        }
        StuTheoryRecordDto stuTheoryRecordDto = new StuTheoryRecordDto();
        stuTheoryRecordDto.setExamCount(stuTheoryRecord.getExamCount());
        stuTheoryRecordDto.setHighestScore(stuTheoryRecord.getHighestScore());
        stuTheoryRecordDto.setLowestScore(stuTheoryRecord.getLowestScore());
        stuTheoryRecordDto.setAverageScore(stuTheoryRecord.getAverageScore());

        //设置权重
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if(totalWeight==null){
            stuTheoryRecordDto.setWeight(BigDecimal.valueOf(0.2));
        }else {
            stuTheoryRecordDto.setWeight(totalWeight.getLlkh());
        }
        return new ResultEntity<>(HttpStatus.OK, "学生端-成绩中心-理论考核成绩展示",stuTheoryRecordDto);
    }


    //学生端 成绩总览
    @AnonymousAccess
    @ApiOperation("学生端-成绩中心-成绩总览展示")
    @PostMapping("selectStuTotalScore")
    public ResultEntity<List<StuTotalScoreDto>> selectStuTotalScore(@RequestParam String userId, @RequestParam String schoolId) {
        List<StuTotalScoreDto> returnDtoList=new ArrayList<>();
        //客户案例数据
        StuTotalScoreDto clientCaseReturnDto = new StuTotalScoreDto();
        List<StuClientCaseScoreDto> clientCaseList=trainingService.selectStuClientCaseScoreList(userId,schoolId);
        clientCaseReturnDto.setModelName("客户案例成绩");
        BigDecimal clientCaseTrainingTime=BigDecimal.ZERO;
        BigDecimal clientCaseTotalTrainingScore=BigDecimal.ZERO;
        clientCaseReturnDto.setTrainingScore(BigDecimal.ZERO);
        clientCaseReturnDto.setTrainingTotalScore(BigDecimal.ZERO);
        clientCaseReturnDto.setUseTime(BigDecimal.ZERO);
        //根据学校iD获取成绩总览权重
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if(totalWeight==null){
            totalWeight=new TotalWeight();
            totalWeight.setKhan(BigDecimal.valueOf(0.4));
            totalWeight.setZhgh(BigDecimal.valueOf(0.4));
            totalWeight.setLlkh(BigDecimal.valueOf(0.2));
        }
        clientCaseReturnDto.setScoreWight(totalWeight.getKhan());
        if(clientCaseList.isEmpty()){
            returnDtoList.add(clientCaseReturnDto);
        }else {
            for (int i = 0; i < clientCaseList.size(); i++) {
                StuClientCaseScoreDto clientCase = clientCaseList.get(i);
                BigDecimal trainingTime = clientCase.getTrainingTime()==null?BigDecimal.ZERO:clientCase.getTrainingTime();
                BigDecimal totalScore = clientCase.getTrainingTotalScore() == null ? BigDecimal.ZERO : clientCase.getTrainingTotalScore();
                clientCaseTrainingTime=clientCaseTrainingTime.add(trainingTime);
                clientCaseTotalTrainingScore=clientCaseTotalTrainingScore.add(totalScore);
            }
            clientCaseReturnDto.setTrainingTotalScore(clientCaseTotalTrainingScore);
            clientCaseReturnDto.setUseTime(clientCaseTrainingTime);
            clientCaseReturnDto.setTrainingScore(clientCaseTotalTrainingScore.multiply(totalWeight.getKhan()));
            returnDtoList.add(clientCaseReturnDto);
        }

        //综合规划成绩
        StuTotalScoreDto synthesisPlanReturnDto = new StuTotalScoreDto();
        synthesisPlanReturnDto.setModelName("综合规划成绩");
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        scoreRankExample.createCriteria().andUseridEqualTo(userId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);
        if(scoreRankList.isEmpty()){
            synthesisPlanReturnDto.setTrainingScore(BigDecimal.ZERO);
            synthesisPlanReturnDto.setUseTime(BigDecimal.ZERO);
            synthesisPlanReturnDto.setTrainingTotalScore(BigDecimal.ZERO);
        }else {
            ScoreRank scoreRank = scoreRankList.get(0);
            Integer totalUseTime=synthesisPlanClientMapper.getTotalUseTime(userId);
            synthesisPlanReturnDto.setUseTime(BigDecimal.valueOf(totalUseTime));
            synthesisPlanReturnDto.setTrainingScore(scoreRank.getZhghScore().multiply(totalWeight.getZhgh()));
            synthesisPlanReturnDto.setTrainingTotalScore(scoreRank.getZhghScore());
        }
        synthesisPlanReturnDto.setScoreWight(totalWeight.getZhgh());
        returnDtoList.add(synthesisPlanReturnDto);

        //理论考试成绩
        StuTotalScoreDto theoryReturnDto = new StuTotalScoreDto();
        if(scoreRankList.isEmpty()){
            theoryReturnDto.setTrainingScore(BigDecimal.ZERO);
            theoryReturnDto.setUseTime(BigDecimal.ZERO);
            theoryReturnDto.setTrainingTotalScore(BigDecimal.ZERO);
        }else {
            StuTheoryRecord stuTheoryRecord = stuTheoryRecordMapper.selectByPrimaryKey(userId);
            if(stuTheoryRecord==null){
                theoryReturnDto.setTrainingScore(BigDecimal.ZERO);
                theoryReturnDto.setUseTime(BigDecimal.ZERO);
                theoryReturnDto.setTrainingTotalScore(BigDecimal.ZERO);
            }else {
                theoryReturnDto.setUseTime(stuTheoryRecord.getTotalDuration());
                theoryReturnDto.setTrainingTotalScore(stuTheoryRecord.getAverageScore());
                theoryReturnDto.setTrainingTotalScore(stuTheoryRecord.getAverageScore().multiply(totalWeight.getLlkh()));
            }
        }
        theoryReturnDto.setScoreWight(totalWeight.getLlkh());
        returnDtoList.add(synthesisPlanReturnDto);
        return new ResultEntity<>(HttpStatus.BAD_REQUEST,"学生端-成绩中心-成绩总览展示成功" ,returnDtoList);
    }


    //教师端 客户案例成绩查询 成绩总览成绩查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-成绩总览展示、客户案例展示  客户案例总成绩为客户案例成绩")
    @PostMapping("selectTeaSummaryOfResults")
    public ResultEntity<PageInfo<ScoreRank>> selectTeaSummaryOfResults(@RequestParam String schoolId,@RequestParam Integer index,@RequestParam Integer size,
                                                  @RequestParam(required = false) String name,
                                                  @RequestParam(required = false) String studentId,
                                                  @RequestParam(required = false) String classId){
        PageHelper.startPage(index, size);
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        if(StringUtils.isNotBlank(name)){
            criteria.andNameEqualTo(name);
        }
        if(StringUtils.isNotBlank(studentId)){
            criteria.andSchoolIdEqualTo(studentId);
        }
        if(StringUtils.isNotBlank(classId)){
            criteria.andClassIdEqualTo(classId);
        }
        scoreRankExample.setOrderByClause("khal_score+zhgh_score+llkh_score desc");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);
        //查询成绩总览权重
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if(totalWeight==null){
            totalWeight.setKhan(BigDecimal.valueOf(0.4));
            totalWeight.setZhgh(BigDecimal.valueOf(0.4));
            totalWeight.setLlkh(BigDecimal.valueOf(0.2));
        }
        //设置总成绩
        for (int i = 0; i < scoreRankList.size(); i++) {
            ScoreRank scoreRank = scoreRankList.get(i);
            scoreRank.setTotalScore((totalWeight.getKhan().multiply(scoreRank.getKhalScore()))
                    .add(totalWeight.getZhgh().multiply(scoreRank.getZhghScore()))
                    .add(totalWeight.getLlkh().multiply(scoreRank.getLlkhScore())));
        }
        PageInfo pageInfo = new PageInfo(scoreRankList);
        return new ResultEntity<PageInfo<ScoreRank>>(pageInfo);
    }

    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-成绩总览权重查询")
    @PostMapping("seleteTotalWeight")
    public ResultEntity<TotalWeight> seleteTotalWeight(@RequestBody String schoolId){
        TotalWeight totalWeight = totalWeightMapper.selectByPrimaryKey(schoolId);
        if(totalWeight==null){
            totalWeight.setSchoolId(schoolId);
            totalWeight.setKhan(BigDecimal.valueOf(0.4));
            totalWeight.setZhgh(BigDecimal.valueOf(0.4));
            totalWeight.setLlkh(BigDecimal.valueOf(0.2));
        }
        return new ResultEntity<>(totalWeight);
    }

    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-成绩总览权重设置")
    @PostMapping("setTotalWeight")
    public Boolean setTotalWeight(@RequestBody TotalWeight totalWeight){
        TotalWeight totalWeight1 = totalWeightMapper.selectByPrimaryKey(totalWeight.getSchoolId());
        if(totalWeight1==null){
            int insert = totalWeightMapper.insert(totalWeight);
            return insert==1;
        }else {
            int i = totalWeightMapper.updateByPrimaryKey(totalWeight);
            return i==1;
        }
    }



    //教师端 成绩总览导出
    @AnonymousAccess
    @GetMapping("/exportSummaryOfResults")
    @ApiOperation("教师端--成绩总览导出页面导出")
    public void exportSummaryOfResults(HttpServletResponse response,
                           @RequestParam String schoolId) {
        List<ScoreRank> scoreRankList=scoreRankService.selectExportScoreRankList(schoolId);
        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"排名", "姓名", "学号", "总成绩", "班级", "客户案例成绩", "综合规划成绩", "理论考核成绩"};

        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("rankId", "name", "studentId", "totalScore", "classId", "khalScore", "zhghScore","llkhScore");
        try {
            FilePortUtil.exportExcel(response, title, headers, scoreRankList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    //客户案例成绩查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-成绩总览展示、客户案例展示  客户案例总成绩为客户案例成绩")
    @PostMapping("selectTeaClientCaseScoreList")
    public ResultEntity<PageInfo<ScoreRank>> selectTeaClientCaseScoreList(@RequestParam String schoolId,@RequestParam Integer index,@RequestParam Integer size,
                                                                       @RequestParam(required = false) String name,
                                                                       @RequestParam(required = false) String studentId,
                                                                       @RequestParam(required = false) String classId){
        PageHelper.startPage(index, size);
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        if(StringUtils.isNotBlank(name)){
            criteria.andNameEqualTo(name);
        }
        if(StringUtils.isNotBlank(studentId)){
            criteria.andSchoolIdEqualTo(studentId);
        }
        if(StringUtils.isNotBlank(classId)){
            criteria.andClassIdEqualTo(classId);
        }
        scoreRankExample.setOrderByClause("khal_score desc");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);

        //设置总成绩
        for (int i = 0; i < scoreRankList.size(); i++) {
            ScoreRank scoreRank = scoreRankList.get(i);
            scoreRank.setTotalScore(scoreRank.getZhghScore());
        }
        PageInfo pageInfo = new PageInfo(scoreRankList);
        return new ResultEntity<PageInfo<ScoreRank>>(pageInfo);
    }

    //教师端 客户案例导出
    @AnonymousAccess
    @GetMapping("/exportClientCase")
    @ApiOperation("教师端--客户案例导出页面导出")
    public void exportClientCase(HttpServletResponse response,
                                       @RequestParam String schoolId) {
        List<ScoreRank> scoreRankList=scoreRankService.selectExportScoreRankList(schoolId);
        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"排名", "姓名", "学号", "总成绩", "风险测评分数", "财务分析分数", "现金规划分数","教育规划分数","消费规划分数","创业规划分数","退休规划分数","保险规划分数","投资规划分数","税务筹划分数","财产分配分数","财产传承分数"};
        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("rankId", "name", "studentId", "khalScore", "fxcp", "cwfx","xjgh","jygh","xfgh","cygh","txgh","bxgh","tzgh","swch","ccfp","cccc");
        try {
            FilePortUtil.exportExcel(response, title, headers, scoreRankList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    //教师端 客户案例权重查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-成绩总览权重查询")
    @PostMapping("seleteClientWeight")
    public ResultEntity<ClientWeight> seleteClientWeight(@RequestBody String schoolId){
        ClientWeight clientWeight = clientWeightMapper.selectByPrimaryKey(schoolId);
        if(clientWeight==null){
            clientWeight.setFxcp(BigDecimal.valueOf(0.05));
            clientWeight.setCwfx(BigDecimal.valueOf(0.20));
            clientWeight.setXjgh(BigDecimal.valueOf(0.05));
            clientWeight.setJygh(BigDecimal.valueOf(0.1));
            clientWeight.setXfgh(BigDecimal.valueOf(0.1));
            clientWeight.setCygh(BigDecimal.valueOf(0.05));
            clientWeight.setTxgh(BigDecimal.valueOf(0.1));
            clientWeight.setBxgh(BigDecimal.valueOf(0.15));
            clientWeight.setTzgh(BigDecimal.valueOf(0.05));
            clientWeight.setSwch(BigDecimal.valueOf(0.05));
            clientWeight.setCcfp(BigDecimal.valueOf(0.05));
            clientWeight.setCccc(BigDecimal.valueOf(0.05));
        }
        return new ResultEntity<>(clientWeight);
    }

    //教师端 客户案例权重设置
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-客户案例权重设置")
    @PostMapping("setClientWeight")
    public Boolean setClientWeight(@RequestBody ClientWeight clientWeight){
        ClientWeight clientWeight1 = clientWeightMapper.selectByPrimaryKey(clientWeight.getSchoolId());
        if(clientWeight1==null){
            int insert = clientWeightMapper.insert(clientWeight);
            return insert==1;
        }else {
            int i = clientWeightMapper.updateByPrimaryKey(clientWeight);
            return i==1;
        }
    }


    //教师端 客户案例章节成绩详情
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-客户案例章节成绩详情")
    @PostMapping("selectTeaClientChapter")
    public ResultEntity<StuTrainingDto>selectTeaClientChapter(@RequestParam String userId,
                                                                        @RequestParam String chapterName,
                                                                        @RequestParam String schoolId){
        ChapterExample chapterExample = new ChapterExample();
        chapterExample.createCriteria().andChapterNameEqualTo(chapterName);
        List<Chapter> chapters = chapterMapper.selectByExample(chapterExample);
        ResultEntity<StuTrainingDto> scoreInfo = getScoreInfo(userId, chapters.get(0).getChapterId(), schoolId);
        return scoreInfo;
    }


    //教师端 客户案例实验报告列表查询
    @AnonymousAccess
    @ApiOperation("教师端--客户案例实验报告列表查询")
    @PostMapping("selectTrainingReportList")
    public ResultEntity<PageInfo<TrainingReport>> selectTrainingReportList(@RequestParam Integer index,
                                                                           @RequestParam Integer size,
                                                                           @RequestParam String schoolId) {
        PageHelper.startPage(index, size);
        TrainingReportExample trainingReportExample = new TrainingReportExample();
        trainingReportExample.createCriteria().andSchoolIdEqualTo(schoolId);
        List<TrainingReport> trainingReports = trainingReportMapper.selectByExampleWithBLOBs(trainingReportExample);
        PageInfo pageInfo = new PageInfo(trainingReports);
        return new ResultEntity<PageInfo<TrainingReport>>(pageInfo);
    }


    //教师端 客户案例获取单个学生实训报告
    @GetMapping("/getReportByReportId")
    @ApiOperation("教师端--客户案例获取单个学生实训报告")
    @AnonymousAccess
    public ResponseEntity<FileSystemResource> getReport(@RequestParam String reportId, HttpServletResponse response) {
        TrainingReport trainingReport = trainingReportMapper.selectByPrimaryKey(reportId);
        String fileUrl = trainingReport.getFilePath();
        String videoPath = filePath + fileUrl;
        File videoFile = new File(videoPath);
        if (videoFile.exists()) {
            FileSystemResource fileSystemResource = new FileSystemResource(videoFile);
            return ResponseEntity.ok()
                    .contentType(MediaType.parseMediaType("application/pdf"))
                    .body(fileSystemResource);
        } else {
            return ResponseEntity.notFound().build();
        }
    }

    //教师端 客户案例评阅 学生信息列表查询  *****

    //教师端 客户案例实验报告评阅
    @AnonymousAccess
    @PostMapping("/updateScoreOrComment")
    @ApiOperation("教师端--实训报告评阅页面老师评分或评语")
    public ResultEntity updateWeightByModule(@RequestParam String reportId,
                                             @RequestParam(required = false) BigDecimal score,
                                             @RequestParam(required = false) String comment) {
        TrainingReport trainingReport = trainingReportMapper.selectByPrimaryKey(reportId);
        if (score == null || score.intValue() < 0 || score.intValue() > 100) {
            return new ResultEntity(HttpStatus.BAD_REQUEST, "请输入正确的分数！");
        } else {
            trainingReport.setTeacherScore(score);
        }
        if (StringUtils.isNotBlank(comment)) {
            trainingReport.setTeacherComment(comment);
        }
        trainingReportMapper.updateByPrimaryKey(trainingReport);
        return new ResultEntity(HttpStatus.OK, "修改成功！");
    }


    //教师端 综合规划列表查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-综合规划成绩展示")
    @PostMapping("selectTeaSynthesisPlanScoreList")
    public ResultEntity<PageInfo<ScoreRank>> selectTeaSynthesisPlanScoreList(@RequestParam String schoolId,@RequestParam Integer index,@RequestParam Integer size,
                                                                                        @RequestParam(required = false) String name,
                                                                                        @RequestParam(required = false) String studentId,
                                                                                        @RequestParam(required = false) String classId){
        PageHelper.startPage(index, size);
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        if(StringUtils.isNotBlank(name)){
            criteria.andNameEqualTo(name);
        }
        if(StringUtils.isNotBlank(studentId)){
            criteria.andSchoolIdEqualTo(studentId);
        }
        if(StringUtils.isNotBlank(classId)){
            criteria.andClassIdEqualTo(classId);
        }
        scoreRankExample.setOrderByClause("zhgh_score");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);

        //设置总成绩
        for (int i = 0; i < scoreRankList.size(); i++) {
            ScoreRank scoreRank = scoreRankList.get(i);
            scoreRank.setTotalScore(scoreRank.getZhghScore());
        }
        PageInfo pageInfo = new PageInfo(scoreRankList);
        return new ResultEntity<PageInfo<ScoreRank>>(pageInfo);
    }

    //教师端 综合规划页面导出
    @AnonymousAccess
    @GetMapping("/exportSynthesisPlanScore")
    @ApiOperation("教师端--综合规划页面导出")
    public void exportSynthesisPlanScore(HttpServletResponse response,
                                  @RequestParam String schoolId) {
        ScoreRankExample scoreRankExample = new ScoreRankExample();
        ScoreRankExample.Criteria criteria = scoreRankExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId).andUpdateDateEqualTo(scoreRankService.getNowDate());
        scoreRankExample.setOrderByClause("zhgh_score");
        List<ScoreRank> scoreRankList = scoreRankMapper.selectByExample(scoreRankExample);

        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"排名", "姓名", "学号", "总成绩", "完成案例个数"};
        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("classId", "name", "studentId", "zhghScore", "countCase");
        try {
            FilePortUtil.exportExcel(response, title, headers, scoreRankList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @GetMapping("/getSubmitStatusCaseListByUserId")
    @ApiOperation("教师端-成绩中心-综合规划-案例列表展示")
    @AnonymousAccess
    public ResultEntity<List<SynthesisPlanClient>> getSubmitStatusCaseListByUserId(String userId){
        SynthesisPlanClientExample synthesisPlanClientExample = new SynthesisPlanClientExample();
        List submitStatusList=new ArrayList();
        submitStatusList.add("未审核");
        submitStatusList.add("已审核");
        synthesisPlanClientExample.createCriteria().andUseridEqualTo(userId).andSubmitStatusIn(submitStatusList);
        List<SynthesisPlanClient> synthesisPlanClientList = synthesisPlanClientMapper.selectByExample(synthesisPlanClientExample);
        return new ResultEntity<>(HttpStatus.OK, "案例列表展示成功",synthesisPlanClientList);
    }

    //教师端 综合规划成绩详情查询
    @PostMapping("/getExamPoint")
    @ApiOperation("展示考核点信息/老师端/学生实验实训")
    @AnonymousAccess
    private ResultEntity<Object> getExamPoint(@RequestParam String caseID) {
        return caseService.getExamPoint(caseID,null, "题目答案", "报告");
    }

    //教师端 综合规划老师评分
    private ResultEntity<Object> getExamPoint11( @ApiParam("右侧考核点设置的参数对象") @RequestParam String json,
                                                 @ApiParam("老师或管理员操作传master,学生操作传stu") String operator,
                                                 String caseid) {
//        Gson gson=new Gson();
//        for (int i = 0; i < typeList.size(); i++) {
//            String type = typeList.get(i);
//        }
//        return caseService.getExamPoint(caseID,null, "题目答案", "报告");
        return null;
    }
    //教师端 综合规划一键评分

    //教师端 理论考核查询
    @AnonymousAccess
    @ApiOperation("教师端-成绩中心-理论考核查询")
    @PostMapping("selectTeaTheoryScore")
    public ResultEntity<PageInfo<StuTheoryRecord>> selectTeaTheoryScore( @RequestParam String schoolId,@RequestParam Integer index,@RequestParam Integer size,
                                                                         @RequestParam(required = false) String name,
                                                                         @RequestParam(required = false) String studentId,
                                                                         @RequestParam(required = false) String classId
                                                                         ) {
        PageHelper.startPage(index, size);
        StuTheoryRecordExample theoryRecordExample = new StuTheoryRecordExample();
        StuTheoryRecordExample.Criteria criteria = theoryRecordExample.createCriteria();
        criteria.andSchoolIdEqualTo(schoolId);
        if(StringUtils.isNotBlank(name)){
            criteria.andNameEqualTo(name);
        }
        if(StringUtils.isNotBlank(studentId)){
            criteria.andSchoolIdEqualTo(studentId);
        }
        if(StringUtils.isNotBlank(classId)){
            criteria.andClassIdEqualTo(classId);
        }
        theoryRecordExample.setOrderByClause("average_score desc");
        List<StuTheoryRecord> stuTheoryRecords = stuTheoryRecordMapper.selectByExample(theoryRecordExample);
        if(stuTheoryRecords.isEmpty()){
            return null;
        }
        PageInfo pageInfo = new PageInfo(stuTheoryRecords);
        return new ResultEntity<PageInfo<StuTheoryRecord>>(pageInfo);
    }

    //教师端 理论考核导出
    @AnonymousAccess
    @GetMapping("/exportTheoryScore")
    @ApiOperation("教师端--理论考核页面导出")
    public void exportTheoryScore(HttpServletResponse response,
                                 @RequestParam String schoolId) {
        List<StuTheoryRecord> stuTheoryRecordList =scoreRankService.selectExportTheoryScoreRankList(schoolId);
        //导出的表名
        String title = IdUtil.simpleUUID();
        //表中第一行表头字段
        String[] headers = {"排名", "姓名", "学号", "平均成绩", "班级", "考试次数", "考试最高分","考试最低分","考试合计时长"};
        //具体需要写入excel需要哪些字段，这些字段取自UserReward类，也就是上面的实际数据结果集的泛型
        List<String> listColumn = Arrays.asList("classId", "name", "studentId", "averageScore", "className", "examCount","highestScore","lowestScore","totalDuration");
        try {
            FilePortUtil.exportExcel(response, title, headers, stuTheoryRecordList, listColumn);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


}
