package com.sztzjy.money_management.controller;

import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@Api(tags = "产品中心相关")
@RequestMapping("api/ProductCenter")
public class ProductCenter {
    @Autowired
    FundProductMapper fundProductMapper;
    @Autowired
    RealGoldProductMapper realGoldProductMapper;
    @Autowired
    BankDepositsMapper bankDepositsMapper;
    @Autowired
    InsuranceProMapper insuranceProMapper;
    @Autowired
    p2pProductMapper p2pProductMapper;
    @Autowired
    ObjectiveQuestionMapper objectiveQuestionMapper;

    @PostMapping("selectFundProduct")
    @ApiOperation("基金产品查询")
    @AnonymousAccess
    public ResultEntity<List<FundProduct>> selectFundProduct(@RequestParam(required = false) String param) {
        FundProductExample example = new FundProductExample();
        if (param != null) {
            FundProductExample.Criteria criteria1 = example.createCriteria();
            FundProductExample.Criteria criteria2 = example.createCriteria();
            FundProductExample.Criteria criteria3 = example.createCriteria();
            criteria1.andFundCodeLike("%" + param + "%");
            criteria2.andFundNameLike("%" + param + "%");
            criteria3.andFundTypeLike("%" + param + "%");
            example.or(criteria1);
            example.or(criteria2);
            example.or(criteria3);
        }
        example.setOrderByClause("update_date asc");
        List<FundProduct> fundProducts = fundProductMapper.selectByExample(example);
        return new ResultEntity<>(HttpStatus.OK, "基金产品查询成功", fundProducts);
    }

    @PostMapping("selectBankDeposits")
    @ApiOperation("银行储蓄查询")
    @AnonymousAccess
    public ResultEntity<List<BankDeposits>> selectBankDeposits(@RequestParam(required = false) String bankName, @RequestParam String schoolId) {
        BankDepositsExample example = new BankDepositsExample();
        BankDepositsExample.Criteria criteria = example.createCriteria();
        if (bankName != null) {
            criteria.andBankNameLike("%" + bankName + "%");
        }
        List<BankDeposits> bankDepositsList = bankDepositsMapper.selectByExample(example);
        return new ResultEntity<>(HttpStatus.OK, "银行储蓄查询成功", bankDepositsList);
    }

    @PostMapping("updateBankDeposits")
    @ApiOperation("银行储蓄编辑")
    @AnonymousAccess
    public ResultEntity updateBankDeposits(@RequestParam BankDeposits bankDeposits) {
        bankDepositsMapper.updateByPrimaryKeySelective(bankDeposits);
        return new ResultEntity<>(HttpStatus.OK, "银行储蓄编辑成功");
    }


    @PostMapping("selectP2PProduct")
    @ApiOperation("P2P产品查询")
    @AnonymousAccess
    public ResultEntity<List<p2pProduct>> selectP2PProduct(@RequestParam(required = false) String productName) {
        p2pProductExample example = new p2pProductExample();
        if (productName != null) {
            p2pProductExample.Criteria criteria = example.createCriteria();
            criteria.andP2pNameLike("%" + productName + "%");
        }
        example.setOrderByClause("update_time asc");
        List<p2pProduct> p2pProducts = p2pProductMapper.selectByExample(example);
        return new ResultEntity<>(HttpStatus.OK, "P2P产品查询成功", p2pProducts);
    }


    @PostMapping("selectRealGoldProduct")
    @ApiOperation("实务黄金查询")
    @AnonymousAccess
    public ResultEntity<List<RealGoldProduct>> selectRealGoldProduct() {
        RealGoldProductExample example = new RealGoldProductExample();
        example.setOrderByClause("trading_day asc");
        List<RealGoldProduct> realGoldProducts = realGoldProductMapper.selectByExample(example);
        return new ResultEntity<>(HttpStatus.OK, "实务黄金查询成功", realGoldProducts);
    }

    @PostMapping("selectInsurancePro")
    @ApiOperation("保险产品查询")
    @AnonymousAccess
    public ResultEntity<List<InsurancePro>> selectInsurancePro(@ApiParam("保险子类") @RequestParam(required = false) String kind,
                                                               @ApiParam("搜索参数") @RequestParam(required = false) String name) {
        InsuranceProExample example = new InsuranceProExample();
        InsuranceProExample.Criteria criteria = example.createCriteria();
        if (kind != null) {
            criteria.andKindEqualTo(kind);
        }
        if (name != null) {
            criteria.andNameLike("%" + name + "%");
        }
        List<InsurancePro> insurancePros = insuranceProMapper.selectByExample(example);
        return new ResultEntity<>(HttpStatus.OK, "保险产品查询成功", insurancePros);
    }


}
